=begin
#Aliseeks API

#AliExpress product searching and product details retrieval API. 

OpenAPI spec version: 1.0.0

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 3.3.4

=end

require 'date'

module AliseeksApi
  # Search request for best selling products 
  class BestSellingSearchRequest
    # The time range to look for best selling products. Can be Top best selling products or Weekly best selling products. 
    attr_accessor :range

    # The category to look for best selling products. Only certain categories can be searched for depending on the `range`. When range is \"top\", only the categories [ \"all\", \"fashion\", \"electronics\", \"sports\", \"health_beauty\", \"kids_baby\", \"home_garden\", \"automotive\" ] are supported. When range is \"weekly\", only the categories [ \"woman\", \"men\", \"electronics\", \"sports\", \"health_beauty\", \"kids_baby\", \"automotive\" ] are supported. 
    attr_accessor :category

    # The number of results to skip. Allows for pagination. 
    attr_accessor :skip

    # AliExpress locale to use. 
    attr_accessor :locale

    attr_accessor :currency

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'range' => :'range',
        :'category' => :'category',
        :'skip' => :'skip',
        :'locale' => :'locale',
        :'currency' => :'currency'
      }
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'range' => :'String',
        :'category' => :'String',
        :'skip' => :'Integer',
        :'locale' => :'String',
        :'currency' => :'RealtimeCurrency'
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      if attributes.has_key?(:'range')
        self.range = attributes[:'range']
      else
        self.range = 'top'
      end

      if attributes.has_key?(:'category')
        self.category = attributes[:'category']
      else
        self.category = 'electronics'
      end

      if attributes.has_key?(:'skip')
        self.skip = attributes[:'skip']
      else
        self.skip = 0
      end

      if attributes.has_key?(:'locale')
        self.locale = attributes[:'locale']
      else
        self.locale = 'en_US'
      end

      if attributes.has_key?(:'currency')
        self.currency = attributes[:'currency']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      range_validator = EnumAttributeValidator.new('String', ['top', 'weekly'])
      return false unless range_validator.valid?(@range)
      category_validator = EnumAttributeValidator.new('String', ['all', 'fashion', 'sports', 'health_beauty', 'home_garden', 'kids_baby', 'automotive', 'electronics', 'men', 'women'])
      return false unless category_validator.valid?(@category)
      true
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] range Object to be assigned
    def range=(range)
      validator = EnumAttributeValidator.new('String', ['top', 'weekly'])
      unless validator.valid?(range)
        fail ArgumentError, 'invalid value for "range", must be one of #{validator.allowable_values}.'
      end
      @range = range
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] category Object to be assigned
    def category=(category)
      validator = EnumAttributeValidator.new('String', ['all', 'fashion', 'sports', 'health_beauty', 'home_garden', 'kids_baby', 'automotive', 'electronics', 'men', 'women'])
      unless validator.valid?(category)
        fail ArgumentError, 'invalid value for "category", must be one of #{validator.allowable_values}.'
      end
      @category = category
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          range == o.range &&
          category == o.category &&
          skip == o.skip &&
          locale == o.locale &&
          currency == o.currency
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [range, category, skip, locale, currency].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.openapi_types.each_pair do |key, type|
        if type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        temp_model = AliseeksApi.const_get(type).new
        temp_model.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
